@@ -5,4 +5,4 @@
 from .timediff import time_slice_diffs
 from .tsdiffplot import plot_tsdiffs, plot_tsdiffs_image
 from .screens import screen
-from ...modalities.fmri import pca
+from ..utils import pca
