@@ -3,16 +3,23 @@
 """
 Test functions for formulae
 """
+from __future__ import print_function, absolute_import
+
+from warnings import catch_warnings, simplefilter
 
 import numpy as np
+from numpy.core.records import fromrecords
+
 import sympy
 
-from nipy.fixes.sympy.utilities.lambdify import implemented_function
+from sympy.utilities.lambdify import implemented_function
 
 from .. import formulae as F
 from ..formulae import terms, Term
+from nipy.utils import VisibleDeprecationWarning
 
 from nibabel.py3k import asbytes
+from nibabel.testing import assert_dt_equal
 
 from nose.tools import (assert_true, assert_equal, assert_false,
                         assert_raises)
@@ -37,35 +44,30 @@ def test_terms():
     # but empty arg returns empty tuple
     assert_equal(terms(()), ())
     # Test behavior of deprecated each_char kwarg
-    try:
-        res = terms('abc', each_char=False)
-    except TypeError:
-        return
-    assert_equal(res, Term('abc'))
-    assert_equal(terms('abc', each_char=True), (a, b, c))
+    assert_raises(TypeError, terms, 'abc', each_char=True)
 
 
 def test_getparams_terms():
     t = F.Term('t')
     x, y, z = [sympy.Symbol(l) for l in 'xyz']
-    yield assert_equal, set(F.getparams(x*y*t)), set([x,y])
-    yield assert_equal, set(F.getterms(x*y*t)), set([t])
+    assert_equal(set(F.getparams(x*y*t)), set([x,y]))
+    assert_equal(set(F.getterms(x*y*t)), set([t]))
 
     matrix_expr = np.array([[x,y*t],[y,z]])
-    yield assert_equal, set(F.getparams(matrix_expr)), set([x,y,z])
-    yield assert_equal, set(F.getterms(matrix_expr)), set([t])
+    assert_equal(set(F.getparams(matrix_expr)), set([x,y,z]))
+    assert_equal(set(F.getterms(matrix_expr)), set([t]))
 
 
 def test_formula_params():
     t = F.Term('t')
     x, y = [sympy.Symbol(l) for l in 'xy']
     f = F.Formula([t*x,y])
-    yield assert_equal, set(f.params), set([x,y] + list(f.coefs.values()))
+    assert_equal(set(f.params), set([x,y] + list(f.coefs.values())))
 
 
 def test_contrast1():
     x = F.Term('x')
-    yield assert_equal, x, x+x
+    assert_equal(x, x+x)
     y = F.Term('y')
     z = F.Term('z')
     f = F.Formula([x,y])
@@ -74,10 +76,10 @@ def test_contrast1():
                                     'diff':F.Formula([x-y]),
                                     'sum':F.Formula([x+y]),
                                     'both':F.Formula([x-y,x+y])})
-    yield assert_almost_equal, C['x'], np.array([1,0])
-    yield assert_almost_equal, C['diff'], np.array([1,-1])
-    yield assert_almost_equal, C['sum'], np.array([1,1])
-    yield assert_almost_equal, C['both'], np.array([[1,-1],[1,1]])
+    assert_almost_equal(C['x'], np.array([1,0]))
+    assert_almost_equal(C['diff'], np.array([1,-1]))
+    assert_almost_equal(C['sum'], np.array([1,1]))
+    assert_almost_equal(C['both'], np.array([[1,-1],[1,1]]))
 
     f = F.Formula([x,y,z])
     arr = F.make_recarray([[3,5,4],[8,21,-1],[4,6,-2]], 'xyz')
@@ -85,10 +87,10 @@ def test_contrast1():
                                     'diff':F.Formula([x-y]),
                                     'sum':F.Formula([x+y]),
                                     'both':F.Formula([x-y,x+y])})
-    yield assert_almost_equal, C['x'], np.array([1,0,0])
-    yield assert_almost_equal, C['diff'], np.array([1,-1,0])
-    yield assert_almost_equal, C['sum'], np.array([1,1,0])
-    yield assert_almost_equal, C['both'], np.array([[1,-1,0],[1,1,0]])
+    assert_almost_equal(C['x'], np.array([1,0,0]))
+    assert_almost_equal(C['diff'], np.array([1,-1,0]))
+    assert_almost_equal(C['sum'], np.array([1,1,0]))
+    assert_almost_equal(C['both'], np.array([[1,-1,0],[1,1,0]]))
 
 
 def test_formula_from_recarray():
@@ -186,24 +188,121 @@ def test_mul():
     f2 = F.Factor('t', [2,3,4])
     t2 = f['t_2']
     x = F.Term('x')
-    yield assert_equal, t2, t2*t2
-    yield assert_equal, f, f*f
-    yield assert_false, f == f2
-    yield assert_equal, set((t2*x).atoms()), set([t2,x])
+    assert_equal(t2, t2*t2)
+    assert_equal(f, f*f)
+    assert_false(f == f2)
+    assert_equal(set((t2*x).atoms()), set([t2,x]))
+
+
+def test_factor_add_sub():
+    # Test adding and subtracting Factors
+    f1 = F.Factor('t', [2, 3, 4])
+    f2 = F.Factor('t', [2, 3])
+    # Terms do not cancel in addition
+    assert_equal(f1 + f2, F.Formula(np.hstack((f1.terms, f2.terms))))
+    assert_equal(f1 - f2, F.Factor('t', [4]))
+    f3 = F.Factor('p', [0, 1])
+    assert_equal(f1 + f3, F.Formula(np.hstack((f1.terms, f3.terms))))
+    assert_equal(f1 - f3, f1)
+
+
+def test_term_order_sub():
+    # Test preservation of term order in subtraction
+    f1 = F.Formula(terms('z, y, x, w'))
+    f2 = F.Formula(terms('x, y, a'))
+    assert_array_equal((f1 - f2).terms, terms('z, w'))
+    assert_array_equal((f2 - f1).terms, terms('a'))
+
+
+def assert_starr_equal(a, b):
+    assert_equal(a.shape, b.shape)
+    assert_equal(a.dtype.names, b.dtype.names)
+    for name in a.dtype.names:
+        assert_array_equal(a[name], b[name])
+        assert_dt_equal(a[name].dtype, b[name].dtype)
 
 
 def test_make_recarray():
-    m = F.make_recarray([[3,4],[4,6],[7,9]], 'wv', [np.float, np.int])
-    assert_equal(m.dtype.names, ('w', 'v'))
+    # Test make_array
+    # From list / sequence
+    # 2D case
+    data_2d = [(3, 4), (4, 6), (7, 9)]
+    m = F.make_recarray(data_2d, 'wv', [np.float, np.int])
+    assert_starr_equal(m, fromrecords(
+        data_2d, dtype=[('w', float), ('v', int)]))
+    # 1D vector, sequence and array
+    for data_1d in (range(4), np.arange(4).astype(float)):
+        # Column vector. For array case, drop name dim for shape match
+        assert_starr_equal(
+            F.make_recarray(data_1d, ['f1'], drop_name_dim=True),
+            np.arange(4).astype([('f1', float)]))
+        # Row vector. Drop name dim for shape match
+        assert_starr_equal(
+            F.make_recarray(data_1d, 'abcd', drop_name_dim=True),
+            np.array(tuple(range(4)), dtype=[(c, float) for c in 'abcd']))
+    # From another recarray, reaming fields
     m2 = F.make_recarray(m, 'xy')
-    assert_equal(m2.dtype.names, ('x', 'y'))
+    assert_starr_equal(m2, fromrecords(
+        data_2d, dtype=[('x', float), ('y', int)]))
+    # Recarrays don't change shape, trailing dimensions or no
+    assert_starr_equal(F.make_recarray(m2, 'xy'), m2)
+    m2_dash = np.reshape(m2, (3, 1, 1, 1))
+    assert_starr_equal(F.make_recarray(m2_dash, 'xy'), m2_dash)
+    # From an array, drop dim case
+    arr = np.array(data_2d)
+    assert_equal(arr.shape, (3, 2))
+    assert_starr_equal(
+        F.make_recarray(arr, 'xy', drop_name_dim=True),
+        fromrecords(data_2d, dtype=[('x', int), ('y', int)]))
+    assert_starr_equal(
+        F.make_recarray(arr.astype(float), 'xy', drop_name_dim=True),
+        fromrecords(data_2d, dtype=[('x', float), ('y', float)]))
+    assert_starr_equal(
+        F.make_recarray(arr.reshape((3, 1, 2)), 'xy', drop_name_dim=True),
+        fromrecords(data_2d, dtype=[('x', int), ('y', int)]).
+        reshape((3, 1)))
+    # Not drop dim case, trailing length 1 axis.
+    assert_starr_equal(
+        F.make_recarray(arr, 'xy', drop_name_dim=False),
+        fromrecords(data_2d, dtype=[('x', int), ('y', int)]).
+        reshape((3, 1)))
+    assert_starr_equal(
+        F.make_recarray(arr.reshape((3, 1, 2)), 'xy', drop_name_dim=False),
+        fromrecords(data_2d, dtype=[('x', int), ('y', int)]).
+        reshape((3, 1, 1)))
+    # False case is the default, with warning (for now)
+    with catch_warnings(record=True) as warn_list:
+        # Clear any pre-existing warnings cached in formula module, to make
+        # sure warning is triggered..  See
+        # nibabel.testing.clear_and_catch_warnings for detail.
+        if hasattr(F, '__warningregistry__'):
+            F.__warningregistry__.clear()
+        simplefilter('always')
+        assert_starr_equal(
+            F.make_recarray(arr, 'xy'),
+            fromrecords(data_2d, dtype=[('x', int), ('y', int)]).
+            reshape((3, 1)))
+        assert_equal(warn_list[0].category, VisibleDeprecationWarning)
+    # Can't pass dtypes to array version of function
+    assert_raises(ValueError, F.make_recarray, arr, 'xy', [int, float])
+
+
+def test_make_recarray_axes():
+    # On earlier numpy, axis to which names applied depends on memory layout
+    # C contiguous
+    arr = np.arange(9).reshape((3,3))
+    s_arr = F.make_recarray(arr, 'abc', drop_name_dim=True)
+    assert_array_equal(s_arr['a'], arr[:, 0])
+    # Fortran contiguous
+    s_arr = F.make_recarray(arr.T, 'abc', drop_name_dim=True)
+    assert_array_equal(s_arr['a'], arr[0, :])
 
 
 def test_str_formula():
     t1 = F.Term('x')
     t2 = F.Term('y')
     f = F.Formula([t1, t2])
-    yield assert_equal, str(f), "Formula([x, y])"
+    assert_equal(str(f), "Formula([x, y])")
 
 
 def test_design():
@@ -282,13 +381,13 @@ def test_alias():
 
 def test_factor_getterm():
     fac = F.Factor('f', 'ab')
-    yield assert_equal, fac['f_a'], fac.get_term('a')
+    assert_equal(fac['f_a'], fac.get_term('a'))
     fac = F.Factor('f', [1,2])
-    yield assert_equal, fac['f_1'], fac.get_term(1)
+    assert_equal(fac['f_1'], fac.get_term(1))
     fac = F.Factor('f', [1,2])
-    yield assert_raises, ValueError, fac.get_term, '1'
+    assert_raises(ValueError, fac.get_term, '1')
     m = fac.main_effect
-    yield assert_equal, set(m.terms), set([fac['f_1']-fac['f_2']])
+    assert_equal(set(m.terms), set([fac['f_1']-fac['f_2']]))
 
 
 def test_stratify():
@@ -296,7 +395,7 @@ def test_stratify():
 
     y = sympy.Symbol('y')
     f = sympy.Function('f')
-    yield assert_raises, ValueError, fac.stratify, f(y)
+    assert_raises(ValueError, fac.stratify, f(y))
 
 
 def test_nonlin1():
@@ -321,7 +420,7 @@ def test_nonlin1():
     n = F.make_recarray([(2,3,'a'),(4,5,'b'),(5,6,'a')], 'xyf', ['d','d','S1'])
     p = F.make_recarray([1,2,3,4], ['_x0', '_x1', '_b0', '_b1'])
     A = f.design(n, p)
-    print A, A.dtype
+    print(A, A.dtype)
 
 
 def test_intercept():
@@ -336,7 +435,7 @@ def test_nonlin2():
     t = sympy.Symbol('th')
     p = F.make_recarray([3], ['tt'])
     f = F.Formula([sympy.exp(t*z)])
-    yield assert_raises, ValueError, f.design, dz, p
+    assert_raises(ValueError, f.design, dz, p)
 
 
 def test_Rintercept():
@@ -353,9 +452,9 @@ def test_return_float():
     f = F.Formula([x,x**2])
     xx= F.make_recarray(np.linspace(0,10,11), 'x')
     dtype = f.design(xx).dtype
-    yield assert_equal, set(dtype.names), set(['x', 'x**2'])
+    assert_equal(set(dtype.names), set(['x', 'x**2']))
     dtype = f.design(xx, return_float=True).dtype
-    yield assert_equal, dtype, np.float
+    assert_equal(dtype, np.float)
 
 
 def test_subtract():
@@ -363,10 +462,10 @@ def test_subtract():
     f1 = F.Formula([x,y])
     f2 = F.Formula([x,y,z])
     f3 = f2 - f1
-    yield assert_equal, set(f3.terms), set([z])
+    assert_equal(set(f3.terms), set([z]))
     f4 = F.Formula([y,z])
     f5 = f1 - f4
-    yield assert_equal, set(f5.terms), set([x])
+    assert_equal(set(f5.terms), set([x]))
 
 
 def test_subs():
@@ -375,7 +474,7 @@ def test_subs():
     z = F.Term('z')
     f = F.Formula([t1, t2])
     g = f.subs(t1, z)
-    yield assert_equal, list(g.terms), [z, t2]
+    assert_equal(list(g.terms), [z, t2])
 
 
 def test_natural_spline():
@@ -385,24 +484,24 @@ def test_natural_spline():
     xx= F.make_recarray(np.linspace(0,10,101), 'x')
     dd=ns.design(xx, return_float=True)
     xx = xx['x']
-    yield assert_almost_equal, dd[:,0], xx
-    yield assert_almost_equal, dd[:,1], xx**2
-    yield assert_almost_equal, dd[:,2], xx**3
-    yield assert_almost_equal, dd[:,3], (xx-2)**3*np.greater_equal(xx,2)
-    yield assert_almost_equal, dd[:,4], (xx-6)**3*np.greater_equal(xx,6)
-    yield assert_almost_equal, dd[:,5], (xx-9)**3*np.greater_equal(xx,9)
+    assert_almost_equal(dd[:,0], xx)
+    assert_almost_equal(dd[:,1], xx**2)
+    assert_almost_equal(dd[:,2], xx**3)
+    assert_almost_equal(dd[:,3], (xx-2)**3*np.greater_equal(xx,2))
+    assert_almost_equal(dd[:,4], (xx-6)**3*np.greater_equal(xx,6))
+    assert_almost_equal(dd[:,5], (xx-9)**3*np.greater_equal(xx,9))
 
     ns=F.natural_spline(xt, knots=[2,9,6], intercept=True)
     xx= F.make_recarray(np.linspace(0,10,101), 'x')
     dd=ns.design(xx, return_float=True)
     xx = xx['x']
-    yield assert_almost_equal, dd[:,0], 1
-    yield assert_almost_equal, dd[:,1], xx
-    yield assert_almost_equal, dd[:,2], xx**2
-    yield assert_almost_equal, dd[:,3], xx**3
-    yield assert_almost_equal, dd[:,4], (xx-2)**3*np.greater_equal(xx,2)
-    yield assert_almost_equal, dd[:,5], (xx-9)**3*np.greater_equal(xx,9)
-    yield assert_almost_equal, dd[:,6], (xx-6)**3*np.greater_equal(xx,6)
+    assert_almost_equal(dd[:,0], 1)
+    assert_almost_equal(dd[:,1], xx)
+    assert_almost_equal(dd[:,2], xx**2)
+    assert_almost_equal(dd[:,3], xx**3)
+    assert_almost_equal(dd[:,4], (xx-2)**3*np.greater_equal(xx,2))
+    assert_almost_equal(dd[:,5], (xx-9)**3*np.greater_equal(xx,9))
+    assert_almost_equal(dd[:,6], (xx-6)**3*np.greater_equal(xx,6))
 
 
 def test_factor_term():
