@@ -1,17 +1,19 @@
+from __future__ import absolute_import
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+from itertools import product
+
 import numpy as np
 
-
-from nipy.core.api import (AffineTransform, Image,  
-                           ArrayCoordMap, compose)
+from nipy.core.api import (CoordinateMap, AffineTransform, Image,
+        ArrayCoordMap, vox2mni)
 from nipy.core.reference import slices
 from nipy.algorithms.resample import resample, resample_img2img
 from nipy.io.api import load_image
 
 from nose.tools import assert_true, assert_raises
 
-from numpy.testing import assert_array_almost_equal
+from numpy.testing import assert_array_almost_equal, assert_array_equal
 from nipy.testing import funcfile, anatfile
 
 
@@ -19,7 +21,7 @@ def test_resample_img2img():
     fimg = load_image(funcfile)
     aimg = load_image(anatfile)
     resimg = resample_img2img(fimg, fimg)
-    yield assert_true, np.allclose(np.asarray(resimg), np.asarray(fimg))
+    yield assert_true, np.allclose(resimg.get_data(), fimg.get_data())
     yield assert_raises, ValueError, resample_img2img, fimg, aimg
 
 
@@ -171,6 +173,47 @@ def test_resample3d():
     assert_array_almost_equal(ir.get_data()[44:49,32:47,20:23], 3.)
 
 
+def test_resample_outvalue():
+    # Test resampling with different modes, constant values, datatypes, orders
+
+    def func(xyz):
+        return xyz + np.asarray([1,0,0])
+
+    coordmap =  vox2mni(np.eye(4))
+    arr = np.arange(3 * 3 * 3).reshape(3, 3, 3)
+    aff = np.eye(4)
+    aff[0, 3] = 1.  # x translation
+    for mapping, dt, order in product(
+        [aff, func],
+        [np.int8, np.intp, np.int32, np.int64, np.float32, np.float64],
+        [0, 1, 3]):
+        img = Image(arr.astype(dt), coordmap)
+        # Test constant value of 0
+        img2 = resample(img, coordmap, mapping, img.shape,
+                        order=order, mode='constant', cval=0.)
+        exp_arr = np.zeros(arr.shape)
+        exp_arr[:-1, :, :] = arr[1:, :, :]
+        assert_array_almost_equal(img2.get_data(), exp_arr)
+        # Test constant value of 1
+        img2 = resample(img, coordmap, mapping, img.shape,
+                        order=order, mode='constant', cval=1.)
+        exp_arr[-1, :, :] = 1
+        assert_array_almost_equal(img2.get_data(), exp_arr)
+        # Test nearest neighbor
+        img2 = resample(img, coordmap, mapping, img.shape,
+                        order=order, mode='nearest')
+        exp_arr[-1, :, :] = arr[-1, :, :]
+        assert_array_almost_equal(img2.get_data(), exp_arr)
+    # Test img2img
+    target_coordmap = vox2mni(aff)
+    target = Image(arr, target_coordmap)
+    img2 = resample_img2img(img, target, 3, 'nearest')
+    assert_array_almost_equal(img2.get_data(), exp_arr)
+    img2 = resample_img2img(img, target, 3, 'constant', cval=1.)
+    exp_arr[-1, :, :] = 1
+    assert_array_almost_equal(img2.get_data(), exp_arr)
+
+
 def test_nonaffine():
     # resamples an image along a curve through the image.
     #
@@ -198,7 +241,7 @@ def test_nonaffine():
         pylab.gca().set_ylim([0,99])
         pylab.gca().set_xlim([0,89])
         pylab.figure(num=4)
-        pylab.plot(np.asarray(ir))
+        pylab.plot(ir.get_data())
 
 
 def test_2d_from_3d():
