@@ -13,9 +13,10 @@ This technique can be used to reduce the dimensionality of the data.
 More specifically, the data is projected onto the eigenvectors of the
 covariance matrix.
 """
+from __future__ import absolute_import
 
 import numpy as np
-import scipy.linalg as spl
+import numpy.linalg as npl
 
 from ...core.image.image import rollimg
 from ...core.reference.coordinate_map import (io_axis_indices, orth_axes,
@@ -130,7 +131,7 @@ def pca(data, axis=0, mask=None, ncomp=None, standardize=True,
     elif design_resid is None:
         def project_resid(Y): return Y
     else: # matrix passed, we hope
-        projector = np.dot(design_resid, spl.pinv(design_resid))
+        projector = np.dot(design_resid, npl.pinv(design_resid))
         def project_resid(Y):
             return Y - np.dot(projector, Y)
     if standardize:
@@ -156,9 +157,9 @@ def pca(data, axis=0, mask=None, ncomp=None, standardize=True,
     if design_keep is None:
         X = np.eye(data.shape[0])
     else:
-        X = np.dot(design_keep, spl.pinv(design_keep))
+        X = np.dot(design_keep, npl.pinv(design_keep))
     XZ = project_resid(X)
-    UX, SX, VX = spl.svd(XZ, full_matrices=0)
+    UX, SX, VX = npl.svd(XZ, full_matrices=0)
     # The matrix UX has orthonormal columns and represents the
     # final "column space" that the data will be projected onto.
     rank = (SX/SX.max() > tol_ratio).sum()
@@ -169,7 +170,7 @@ def pca(data, axis=0, mask=None, ncomp=None, standardize=True,
     C_full_rank  = _get_covariance(data, UX, rmse_scales_func, mask)
     # find the eigenvalues D and eigenvectors Vs of the covariance
     # matrix
-    D, Vs = spl.eigh(C_full_rank)
+    D, Vs = npl.eigh(C_full_rank)
     # Compute basis vectors in original column space
     basis_vectors = np.dot(UX.T, Vs).T
     # sort both in descending order of eigenvalues
@@ -199,7 +200,7 @@ def _get_covariance(data, UX, rmse_scales_func, mask):
     rank, n_pts = UX.shape
     C = np.zeros((rank, rank))
     # nan_to_num only for floating point masks
-    if not mask is None:
+    if mask is not None:
         nan_to_num = mask.dtype.type in (np.sctypes['float'] +
                                          np.sctypes['complex'])
     # loop over next dimension to save memory
@@ -248,7 +249,7 @@ def pca_image(img, axis='t', mask=None, ncomp=None, standardize=True,
     ----------
     img : Image
         The image on which to perform PCA over the given `axis`
-    axis : str or int
+    axis : str or int, optional
         Axis over which to perform PCA. Default is 't'. If `axis` is an integer,
         gives the index of the input (domain) axis of `img`. If `axis` is a str, can be
         an input (domain) name, or an output (range) name, that maps to an input
